/*
	deck.c
	Copyright (C) 2001 David Fifield

	This file is part of solitairecrypt.

	solitairecrypt is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2, or (at
	your option) any later version.

	solitairecrypt is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "deck.h"

deck *deck_new(void)
{
	deck *d;

	d = (deck *) malloc(sizeof(deck));
	if (d == NULL)
		return NULL;
	deck_init(d);

	return d;
}

void deck_init(deck *d)
{
	card c;

	for (c = 0; c < DECK_SIZE; c++)
		d->cards[c] = c;
	d->joker_a = DECK_SIZE - 2;
	d->joker_b = DECK_SIZE - 1;

	return;
}

void deck_free(deck *d)
{
	free(d);

	return;
}

void deck_move_jokers(deck *d)
{
	unsigned int i;
	int rot;

	rot = 0;
	i = (d->joker_a + 1) % DECK_SIZE;
	if (i <= d->joker_a)
		rot++;
	if (i == d->joker_b)
		d->joker_b = d->joker_a;
	d->cards[d->joker_a] = d->cards[i];
	d->cards[i] = JOKER_A;
	d->joker_a = i;

	i = (d->joker_b + 1) % DECK_SIZE;
	if (i <= d->joker_b)
		rot++;
	if (i == d->joker_a)
		d->joker_a = d->joker_b;
	d->cards[d->joker_b] = d->cards[i];
	d->cards[i] = JOKER_B;

	d->joker_b = (i + 1) % DECK_SIZE;
	if (d->joker_b <= i)
		rot++;
	if (d->joker_b == d->joker_a)
		d->joker_a = i;
	d->cards[i] = d->cards[d->joker_b];
	d->cards[d->joker_b] = JOKER_B;

	deck_rotate(d, rot);

	return;
}

void deck_triple_cut(deck *d)
{
	static card buffer[DECK_SIZE];
	unsigned int i, j;

	if (d->joker_a < d->joker_b) {
		i = d->joker_a;
		j = d->joker_b + 1;
	} else {
		i = d->joker_b;
		j = d->joker_a + 1;
	}
	d->joker_a += DECK_SIZE - j - i;
	d->joker_b += DECK_SIZE - j - i;
	memmove(buffer, d->cards + j, DECK_SIZE - j);
	memmove(buffer + DECK_SIZE - j, d->cards + i, j - i);
	memmove(d->cards + DECK_SIZE - i, d->cards, i);
	memmove(d->cards, buffer, DECK_SIZE - i);

	return;
}

void deck_count_cut(deck *d)
{
	unsigned int i;

	i = d->cards[DECK_SIZE - 1] + 1;
	if (i > DECK_SIZE - 1)
		i = DECK_SIZE - 1;
	deck_count_cut_num(d, i);

	return;
}

void deck_count_cut_num(deck *d, unsigned int n)
{
	static card buffer[DECK_SIZE];

	if (d->joker_a < DECK_SIZE - 1) {
		if (d->joker_a < n)
			d->joker_a += DECK_SIZE - 1;
		d->joker_a = (d->joker_a - n) % (DECK_SIZE - 1);
	}
	if (d->joker_b < DECK_SIZE - 1) {
		if (d->joker_b < n)
			d->joker_b += DECK_SIZE - 1;
		d->joker_b = (d->joker_b - n) % (DECK_SIZE - 1);
	}
	memmove(buffer, d->cards + n, DECK_SIZE - 1 - n);
	memmove(d->cards + DECK_SIZE - 1 - n, d->cards, n);
	memmove(d->cards, buffer, DECK_SIZE - 1 - n);

	return;
}

void deck_rotate(deck *d, int n)
{
	static card buffer[DECK_SIZE];

	if (n == 0)
		return;
	n %= DECK_SIZE;
	if (n < 0)
		n += DECK_SIZE;
	d->joker_a = (d->joker_a + n) % DECK_SIZE;
	d->joker_b = (d->joker_b + n) % DECK_SIZE;
	memmove(buffer, d->cards + DECK_SIZE - n, n);
	memmove(d->cards + n, d->cards, DECK_SIZE - n);
	memmove(d->cards, buffer, n);

	return;
}

void deck_print(deck *d)
{
	unsigned int i;
	card *ptr;

	ptr = d->cards;
	for (i = 0; i < DECK_SIZE; i++) {
		switch (*ptr) {
			case JOKER_A:
				printf(" A ");
				break;
			case JOKER_B:
				printf(" B ");
				break;
			default:
				printf("%2d ", *ptr + 1);
				break;
		}
		ptr++;
		if (i == 25 || i == 51)
			putchar('\n');
	}
	putchar('\n');

	return;
}

