/*
	solitaire.c
	Copyright (C) 2001 David Fifield

	This file is part of solitairecrypt.

	solitairecrypt is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2, or (at
	your option) any later version.

	solitairecrypt is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <ctype.h>

#include "solitaire.h"
#include "deck.h"

char solitaire_encrypt(deck *d, char c)
{
	card cr;

	do
		solitaire_round(d);
	while ((cr = solitaire_output(d)) >= DECK_SIZE - 2);
	c = toupper(c) - 'A' + 1;
	c = (c + cr) % 26 + 'A';

	return c;
}

char solitaire_decrypt(deck *d, char c)
{
	card cr;

	do
		solitaire_round(d);
	while ((cr = solitaire_output(d)) >= DECK_SIZE - 2);
	c = toupper(c) - 'A' - 1;
	if (c < cr)
		c += DECK_SIZE - 2;
	c = (c - cr) % 26 + 'A';

	return c;
}

void solitaire_round(deck *d)
{
	deck_move_jokers(d);
	deck_triple_cut(d);
	deck_count_cut(d);

	return;
}

card solitaire_output(deck *d)
{
	card c;

	c = d->cards[0] + 1;
	if (c > DECK_SIZE - 1)
		c = DECK_SIZE - 1;
	c = d->cards[(int) c];
	if (c > DECK_SIZE - 2)
		c = DECK_SIZE - 2;

	return c;
}

void solitaire_key(deck *d, const char *keyphrase)
{
	int i;
	char c;

	while ((c = *keyphrase++) != '\0') {
		if (isalpha(c) == 0)
			continue;
		i = toupper(c) - 'A' + 1;
		solitaire_round(d);
		deck_count_cut_num(d, i);
	}

	return;
}

